require 'scripts/strict'

math.randomseed(MOAISim.getDeviceTime())

keyCallbacks = {}
require 'scripts/controls'

require 'rigs/Species'
require 'rigs/Dna'
require 'rigs/Biome'
require 'rigs/Sim'
require 'rigs/GuiBox'


--MOAIDebugLines.setStyle ( MOAIDebugLines.PARTITION_CELLS, 2, 1, 1, 1 )
--MOAIDebugLines.setStyle ( MOAIDebugLines.PARTITION_PADDED_CELLS, 1, 0.5, 0.5, 0.5 )
--MOAIDebugLines.setStyle ( MOAIDebugLines.PROP_WORLD_BOUNDS, 2, 0.75, 0.75, 0.75 )
--MOAIDebugLines.setStyle ( MOAIDebugLines.TEXT_BOX, 2, 0.75, 0.75, 0.75 )


MOAIEnvironment.initialWindowWidth = 800
MOAIEnvironment.initialWindowHeight = 600

MOAISim.openWindow ( "SimEvo - Ludum Dare 24", MOAIEnvironment.initialWindowWidth, MOAIEnvironment.initialWindowHeight )

viewport = MOAIViewport.new ()
viewport:setSize ( MOAIEnvironment.initialWindowWidth, MOAIEnvironment.initialWindowHeight )
viewport:setScale ( MOAIEnvironment.initialWindowWidth, MOAIEnvironment.initialWindowHeight )

partition = MOAIPartition.new ()

camera = MOAICamera2D.new ()

layer = MOAILayer2D.new ()
layer:setViewport ( viewport )
layer:setCamera ( camera )
layer:setPartition ( partition )


-- DNA viewport, layer & camera

dnaPartition = MOAIPartition.new ()
dnaViewport = MOAIViewport.new ()
dnaViewport:setSize ( 300, 300 )
dnaViewport:setScale ( 300, 300 )

dnaBackground = MOAILayer2D.new ()
dnaBackground:setLoc(15, 65)
dnaBackground:setViewport (dnaViewport)
local dnaBgPropQuad = MOAIGfxQuad2D.new ()
dnaBgPropQuad:setTexture ( "img/blackbg.png" )
dnaBgPropQuad:setRect ( -150, -150, 150, 150 )
dnaBgProp = MOAIProp2D.new()
dnaBgProp:setDeck(dnaBgPropQuad)
dnaBackground:insertProp(dnaBgProp)
dnaBackground:setParent(layer)

dnaViewCamera = MOAICamera2D.new ()

dnaViewLayer = MOAILayer2D.new ()
dnaViewLayer:setLoc(0, 0)
dnaViewLayer:setViewport (dnaViewport)
dnaViewLayer:setPartition ( dnaPartition )
dnaViewLayer:setCamera ( dnaViewCamera )
dnaViewLayer:setParent ( dnaBackground )

-- SET LAYERS

MOAIRenderMgr.setRenderTable({layer, dnaBackground, dnaViewLayer})

-- FONT

font = MOAIFont.new ()
charcodes = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 .,:;!?()&/-|'
font:loadFromTTF ( 'stuff/arial-rounded.TTF', charcodes, 30, 72 )

-- GUI

speciesNameBox = MOAITextBox.new ()
speciesNameBox:setFont ( font )
speciesNameBox:setRect ( 0, 0, 300, 25 )
speciesNameBox:setAlignment ( MOAITextBox.CENTER_JUSTIFY )
speciesNameBox:setYFlip ( true )
speciesNameBox:setLoc ( -385, 235 )
speciesNameBox:setString ( '<c:000>Bjshdfjkhsdfk<c>' )
speciesNameBox:setTextSize(20)
layer:insertProp ( speciesNameBox )



topBarImage = MOAIGfxQuad2D.new ()
topBarImage:setTexture ( "img/top-bar.png" )
topBarImage:setRect (0, 0, 480, 30 )
topBar = MOAIProp2D.new ()
topBar:setDeck ( topBarImage )
topBar:setLoc ( -240, 275 )
layer:insertProp(topBar)
topBarTxt = MOAITextBox.new ()
topBarTxt:setFont ( font )
topBarTxt:setRect ( 0, 0, 470, 25 )
topBarTxt:setAlignment ( MOAITextBox.CENTER_JUSTIFY )
topBarTxt:setYFlip ( true )
topBarTxt:setLoc ( -235, 276 )
topBarTxt:setString ( '<c:000>Generation <c:980e0e>43<c>   <c:000>Population <c:115ea5>3453<c>   <c:000>Biomass <c:1b6a12>3453<c>' )
topBarTxt:setTextSize(18)
--topBarTxt:setParent(topBar)
layer:insertProp (topBarTxt)
-- BIG Background

bgImage = MOAIGfxQuad2D.new ()
bgImage:setTexture ( "img/background-fluid.png" )
bgImage:setRect ( -400, -300, 400, 300 )

bgProp = MOAIProp2D.new ()
bgProp:setDeck ( bgImage )
bgProp:setPriority(-10)
layer:insertProp(bgProp)


function round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end

-- Sounds

MOAIUntzSystem.initialize ()

btnSound = MOAIUntzSound.new ()
btnSound:load ( 'stuff/bleep1.wav' )
btnSound:setVolume ( 1 )

-- GUI

statBox1 = initGuiBox(150, 200, 15, 380)
statBox2 = initGuiBox(150, 200, 165, 380)
statBox2.lastline = 0

statBox = {}
statBox.name = statBox1:addText('nammme')
statBox.name:setAlignment ( MOAITextBox.CENTER_JUSTIFY )
statBox.name:setRect ( 0, 0, 300 - 5*2, 20 )

statBox1:addText('Mass')
statBox.size = statBox1:addNum('12')
statBox1:addText('Land mobility')
statBox.landMobility = statBox1:addNum('15')
statBox1:addText('Water mobility')
statBox.waterMobility = statBox1:addNum('15')
statBox1:addText('Air mobility')
statBox.airMobility = statBox1:addNum('15')
statBox1:addText('Reproduction')
statBox.reproduction = statBox1:addNum('15')
--statBox1:addText('Predation')
--statBox.predation = statBox1:addNum('15')
--statBox1:addText('Anti Predation')
--statBox.antiPredation = statBox1:addNum('15')
statBox1:addText('Awareness')
statBox.awareness = statBox1:addNum('33')

statBox2:addText('Energy need')
statBox.consumption = statBox2:addNum('33')
statBox2:addText('Photosynthesis')
statBox.photosynthesis = statBox2:addNum('33')
statBox2:addText('Decomposition')
statBox.detrivoricity = statBox2:addNum('33')
statBox2:addText('Digestion')
statBox.digestion = statBox2:addNum('33')
statBox2:addText('')
statBox2:addText('Mutation')
statBox.mutations = statBox2:addNum('33')

biomeStatBox = initGuiBox(300, 200, 400, 65)
biomeStatBox.name = biomeStatBox:addText('Name')
biomeStatBox.name:setAlignment ( MOAITextBox.CENTER_JUSTIFY )
biomeStatBox:addText('Environments')
biomeStatBox.environmentTypes = biomeStatBox:addNum('')
biomeStatBox:addText('Temperature')
biomeStatBox.temperature = biomeStatBox:addNum('')
biomeStatBox:addText('Nutrients')
biomeStatBox.nutrients = biomeStatBox:addNum('15')
biomeStatBox:addText('Biomass')
biomeStatBox.biomass = biomeStatBox:addNum('15')
biomeStatBox:addText('Detrius')
biomeStatBox.detriusmass = biomeStatBox:addNum('15')
biomeStatBox:addText('Luminosity')
biomeStatBox.luminosity = biomeStatBox:addNum('15')
biomeStatBox:addText('Mobility restriction')
biomeStatBox.mobilityRestriction = biomeStatBox:addNum('15')

biomeNameBox = MOAITextBox.new ()
biomeNameBox:setFont ( font )
biomeNameBox:setRect ( 0, 0, 300, 25 )
biomeNameBox:setAlignment ( MOAITextBox.CENTER_JUSTIFY )
biomeNameBox:setYFlip ( true )
biomeNameBox:setLoc ( 0, 0 )
biomeNameBox:setString ( '<c:000>Biome<c>' )
biomeNameBox:setTextSize(20)
biomeNameBox:setParent(biomeStatBox)
layer:insertProp ( biomeNameBox )


biomeStatBox.migrateBtn = biomeStatBox:addButton('Migrate to...', 100, 195)
MOAIInputMgr.clicks.migrate = biomeStatBox.migrateBtn
MOAIInputMgr.hovers.migrate = biomeStatBox.migrateBtn
local oldHover = biomeStatBox.migrateBtn.hoverIn
biomeStatBox.migrateBtn.hoverIn = function(self)
	oldHover(self)
	sim.selectedBiome = sim.nextBiome
	sim.selectedBiome:update()
end
local oldHover = biomeStatBox.migrateBtn.hoverOut
biomeStatBox.migrateBtn.hoverOut = function(self)
	oldHover(self)
	sim.selectedBiome = sim.biome
	sim.selectedBiome:update()
end
biomeStatBox.migrateBtn.onClickUp = function(self)
	sim.biome = sim.nextBiome
	sim.selectedBiome = sim.biome
	sim.selectedBiome:update()
	sim.nextBiome = generateNeighborBiome(sim.biome.type)
end


scoreBox = initGuiBox(200, 200, 550, 380)
scoreBox.name = scoreBox:addText('<c:68e95e>SimEvo Score')
scoreBox.name:setAlignment ( MOAITextBox.CENTER_JUSTIFY )
scoreBox.name:setTextSize ( 25 )
scoreBox.name:setLoc ( 0, -40 )
scoreBox.name:setRect ( 0, 0, 200, 35 )
scoreBox:addText('')
scoreBox:addText('Top <c:68e95e>biomass')
scoreBox.topBiomass = scoreBox:addNum('0')
scoreBox:addText('Top <c:115ea5>population')
scoreBox.topPopulation = scoreBox:addNum('0')
scoreBox:addText('Top <c:b4be0c>species energy')
scoreBox.topEnergy = scoreBox:addNum('0')
scoreBox:addText('')
scoreBox.totalTxt = scoreBox:addText('Total')
scoreBox.totalTxt:setTextSize ( 20 )
scoreBox.totalTxt:setRect ( 0, 0, 200, 35 )
scoreBox.total = scoreBox:addNum('0')
scoreBox.total:setTextSize ( 20 )
scoreBox.total:setRect ( 0, 0, 200 -10, 35 )
scoreBox.newGame = scoreBox:addButton('New game', 50, 180)
MOAIInputMgr.clicks.newGame = scoreBox.newGame
scoreBox.newGame.onClickUp = function ()
	sim.species:delete()

	sim.biome = generateRandomBiome()
	sim.selectedBiome = sim.biome
	sim.nextBiome = generateNeighborBiome(sim.biome.type)

	sim.species = initSpecies('Ludum Darosaur', dnaViewLayer)
	sim.species:start()

	sim.species:select()
	sim.species:update()
	sim.biome:update()
end

MOAIInputMgr.clicks.randomBiome = biomeStatBox.name
biomeStatBox.name.onClickUp = function ()
	local biome = generateRandomBiome()
	sim.biome = biome
	sim.selectedBiome = biome
	biome:update()
end


genePoolBox = initGuiBox(200, 200, 332, 380)
genePoolBox.name = genePoolBox:addText('Gene Pool')
genePoolBox.name:setAlignment ( MOAITextBox.CENTER_JUSTIFY )
genePoolBox.select = genePoolBox:addButton('Select', 50, 180)
MOAIInputMgr.clicks.selectDna = genePoolBox.select
MOAIInputMgr.hovers.selectDna = genePoolBox.select
MOAIInputMgr.clicks.nextDna = genePoolBox.name
genePoolBox.name.onClickUp = function ()
	genePoolBox:nextDna()
end

genePoolBox.select.onClickUp = function ()
	if (genePoolBox.dna[genePoolBox.currentDna]) then
		sim.species:addDna(genePoolBox.dna[genePoolBox.currentDna])
		genePoolBox.dna[genePoolBox.currentDna] = nil
		--genePoolBox:nextDna()
	end
end
local oldHover = genePoolBox.select.hoverIn
genePoolBox.select.hoverIn = function(self)
	oldHover(self)
	if genePoolBox.dna[genePoolBox.currentDna] and not (genePoolBox.dna[genePoolBox.currentDna] == sim.selectedDna) then
		genePoolBox.oldSelect = sim.selectedDna
		genePoolBox.dna[genePoolBox.currentDna]:select()
	end
end
local oldHover = genePoolBox.select.hoverOut
genePoolBox.select.hoverOut = function(self)
	oldHover(self)
	if (genePoolBox.oldSelect) then
		genePoolBox.oldSelect:select()
	end
end

extinctProp = nil

-- Play button
playButton = MOAIProp2D.new ()
playButton.img = MOAIGfxQuad2D.new ()
playButton.img:setTexture ( "img/play.png" )
playButton.img:setRect ( 0, 0, 25, 25 )
playButton.imgHi = MOAIGfxQuad2D.new ()
playButton.imgHi:setTexture ( "img/play-hi.png" )
playButton.imgHi:setRect ( 0, 0, 25, 25 )
playButton:setDeck ( playButton.imgHi )
playButton:setLoc(300, 275)
MOAIInputMgr.clicks.playButton = playButton
MOAIInputMgr.hovers.playButton = playButton
playButton.onClickUp = function ()
	sim.run = true
	playButton:setDeck(playButton.imgHi)
	pauseButton:setDeck(pauseButton.img)
end
playButton.onHoverIn = function ()
	playButton:setDeck(playButton.imgHi)
end
playButton.onHoverOut = function ()
	if (not sim.run) then
		playButton:setDeck(playButton.img)
	end
end
layer:insertProp(playButton)


-- Pause button
pauseButton = MOAIProp2D.new ()

pauseButton.img = MOAIGfxQuad2D.new ()
pauseButton.img:setTexture ( "img/pause.png" )
pauseButton.img:setRect ( 0, 0, 25, 25 )
pauseButton.imgHi = MOAIGfxQuad2D.new ()
pauseButton.imgHi:setTexture ( "img/pause-hi.png" )
pauseButton.imgHi:setRect ( 0, 0, 25, 25 )
pauseButton:setDeck ( pauseButton.img )
pauseButton:setLoc(270, 275)
MOAIInputMgr.clicks.pauseButton = pauseButton
MOAIInputMgr.hovers.pauseButton = pauseButton
pauseButton.onClickUp = function ()
	sim.run = false
	pauseButton:setDeck(pauseButton.imgHi)
	playButton:setDeck(playButton.img)
end
pauseButton.onHoverIn = function ()
	pauseButton:setDeck(pauseButton.imgHi)
end
pauseButton.onHoverOut = function ()
	if (sim.run) then
		pauseButton:setDeck(pauseButton.img)
	end
end
layer:insertProp(pauseButton)

-- INIT DATA

local species = initSpecies('Ludum Darosaur', dnaViewLayer)

local biome = generateRandomBiome()

sim = initSim(species, biome)

sim.nextBiome = generateNeighborBiome(sim.biome.type)
sim.species:start()

biome:update()


if (false) then

	species.dna.legs = initDna('Legs', species)
	species.dna.legs.size = 1
	species.dna.legs.consumption = 2
	species.dna.legs.mobility = 10
	species.dna.legs.parent = body

	species.dna.claws = initDna('Claws', species)
	species.dna.claws.size = 0
	species.dna.claws.consumption = 1
	species.dna.claws.predation = 3
	species.dna.claws.antiPredation = 1
	species.dna.claws.parent = species.dna.legs

	species.dna.digestion = initDna('Digestion', species)
	species.dna.digestion.size = 1
	species.dna.digestion.consumption = 2
	species.dna.digestion.digestion = 25
	species.dna.digestion.detrivoricity = 10
	species.dna.digestion.parent = body

	species.dna.eyes = initDna('Eyes', species)
	species.dna.eyes.size = 0
	species.dna.eyes.consumption = 3
	species.dna.eyes.awareness = 3
	species.dna.eyes.parent = body
end


sim.genepool = genePoolBox

--sim.genepool:addDna(generateTypeDna('body'))
--sim.genepool:addDna(generateTypeDna('limb'))


species:update()
species:select()


