biomeRandom = function (from, to)
	if true or not (math.ceil(from) == from) or not (math.ceil(to) == to) then
		return from + (math.random()*(to - from))
	else
		return math.random(from, to)
	end
end

table.contains = function (table, element)
	for x , value in pairs(table) do
		if value == element then
			return true
		end
	end
	return false
end

biomeBackgrounds = {}

function initBiome (name)
	local biome = {}

	if (name == nil) then
		name = 'Biome #'.. math.random(1,1000)
	end

	biome.prop = MOAIProp2D.new ()

	biome.name = name

	biome.environmentTypes = {}

	biome.background = 'background-tree'

	biome.stats = {}
	biome.stats.temperature = 0
	biome.stats.nutrients = 0
	biome.stats.decompositionRate = 0.1
	biome.stats.biomass = 0
	biome.stats.biomassProducers = 0.02
	biome.stats.biomassChemoProducers = 0
	biome.stats.biomassDetrivores = 0.02
	biome.stats.biomassDecay = 0.02
	biome.stats.detriusmass = 0
	biome.stats.biomeLoss = 0
	biome.stats.luminosity = 0
	biome.stats.mutability = math.random(0, 100)
	biome.stats.mobilityRestriction = 0



	biome.update = function (this)
		biomeStatBox.name:setString ( '<c:fff>' .. biome.name .. '<c>' )
		biomeStatBox.environmentTypes:setString ( table.concat(biome.environmentTypes, ', '))
		for k, value in pairs(this.stats) do
			if (biomeStatBox[k]) then
				if (k == 'temperature') then
					biomeStatBox[k]:setString('<c:74a7fb>' .. math.ceil(value) .. 'c')
				elseif (k == 'luminosity') then
					biomeStatBox[k]:setString('<c:74a7fb>' .. math.ceil(value*100)/100)
				else
					biomeStatBox[k]:setString('<c:74a7fb>' .. math.ceil(value))
				end
			end
		end

		if (this == sim.biome and (not bgProp.currentBackground or not (bgProp.currentBackground == this.background))) then
			if (not biomeBackgrounds[this.background]) then
				local img = MOAIGfxQuad2D.new ()
				img:setTexture ( 'img/' .. this.background ..'.png' )
				img:setRect ( -400, -300, 400, 300 )

				biomeBackgrounds[this.background] = img
			end
			bgProp:setDeck ( biomeBackgrounds[this.background] )
			bgProp.currentBackground = this.background
		end
	end

	-- Apply all generation effects that happen regardless
	biome.doTick = function (this)
		-- Biomass growth from nutrients + photosynthesis
		biome.stats.biomass = biome.stats.biomass + (biome.stats.biomassProducers*biome.stats.luminosity*biome.stats.nutrients)
		biome.stats.nutrients = biome.stats.nutrients - (biome.stats.biomassProducers*biome.stats.luminosity*biome.stats.nutrients)

		-- Biomass growth from nutrients + chemosynthesis
		biome.stats.biomass = biome.stats.biomass + (biome.stats.biomassChemoProducers*biome.stats.nutrients)
		biome.stats.nutrients = biome.stats.nutrients - (biome.stats.biomassChemoProducers*biome.stats.nutrients)

		-- Biomass growth from detrius
		biome.stats.biomass = biome.stats.biomass + (biome.stats.biomassDetrivores*biome.stats.detriusmass)
		biome.stats.detriusmass = biome.stats.detriusmass - (biome.stats.biomassDetrivores*biome.stats.detriusmass)

		-- Biomass to detrius
		biome.stats.detriusmass = biome.stats.detriusmass + biome.stats.biomass * biome.stats.biomassDecay
		biome.stats.biomass = biome.stats.biomass * (1 - biome.stats.biomassDecay)

		-- Detrius to nutrients
		biome.stats.nutrients = biome.stats.nutrients + biome.stats.detriusmass * biome.stats.decompositionRate
		biome.stats.detriusmass = biome.stats.detriusmass - biome.stats.detriusmass * biome.stats.decompositionRate

		-- Detrius & nutrients loss
		--biome.stats.nutrients = biome.stats.nutrients * (1- biome.stats.biomeLoss)
		--biome.stats.detriusmass = biome.stats.detriusmass * (1- biome.stats.biomeLoss)

	end

	return biome
end

generateRandomBiome = function ()
	local randomGenList = {}
	for k, type in pairs(generateBiome) do
		randomGenList[#randomGenList + 1] = k
	end

	return generateBiome[randomGenList[math.random(1, #randomGenList)]]()
end

generateNeighborBiome = function (type)
	while (true)
	do
		local biome = generateRandomBiome()
		if (table.contains(biome.adjecents, type)) then
			return biome
		end
	end
end


generateBiome = {}
generateBiome.seasurface = function ()
	local biome = initBiome()

	biome.name = 'Salt sea surface'
	biome.type = 'seasurface'
	biome.background = 'background-fluid'
	biome.adjecents = {'seabed', 'seashore', 'sky', 'archipelago'}
	biome.environmentTypes = {'water', 'air'}
	biome.stats.temperature = biomeRandom(0, 30)
	biome.stats.nutrients = 0
	biome.stats.biomass = biomeRandom(1000, 100000)
	biome.stats.biomassDecay = biomeRandom(0.05, 0.1)
	biome.stats.biomeLoss = biomeRandom(0.3, 0.5)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0.4, 0.8)
	biome.stats.mutability = biomeRandom(0, 100)
	biome.stats.mobilityRestriction = biomeRandom(0, 2)

	return biome
end

generateBiome.seabed = function ()
	local biome = initBiome()

	biome.name = 'Salt seabed'
	biome.type = 'seabed'
	biome.background = 'background-fluid'
	biome.adjecents = {'seasurface', 'deepseabed', 'seashore'}
	biome.environmentTypes = {'water', 'land'}
	biome.stats.temperature = biomeRandom(0, 15)
	biome.stats.nutrients = biomeRandom(10000, 100000)
	biome.stats.biomass = biomeRandom(2000, 50000)
	biome.stats.biomassDecay = biomeRandom(0.02, 0.03)
	biome.stats.biomeLoss = biomeRandom(0, 0.2)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.8, 1.2)
	biome.stats.luminosity = biomeRandom(0.3, 0.4)
	biome.stats.mutability = biomeRandom(0, 40)
	biome.stats.mobilityRestriction = biomeRandom(0, 2)

	return biome
end

generateBiome.deepseabed = function ()
	local biome = generateBiome.seabed()

	biome.name = 'Deep ocean bed'
	biome.type = 'deepseabed'
	biome.background = 'background-fluid'
	biome.adjecents = {'seabed'}
	biome.environmentTypes = {'water', 'land'}
	biome.stats.temperature = biomeRandom(0, 3)
	biome.stats.nutrients = biomeRandom(10000, 100000)
	biome.stats.biomass = biomeRandom(1000, 10000)
	biome.stats.biomassDecay = biomeRandom(0.05, 0.1)
	biome.stats.biomeLoss = biomeRandom(0, 0.1)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.2)
	biome.stats.luminosity = biomeRandom(0, 0.1)
	biome.stats.mutability = biomeRandom(0, 100)
	biome.stats.mobilityRestriction = biomeRandom(0, 4)

	return biome
end

generateBiome.seashore = function ()
	local biome = initBiome()

	biome.name = 'Sea shore'
	biome.type = 'seashore'
	biome.background = 'background-fluid'
	biome.environmentTypes = {'water', 'air', 'land'}
	biome.adjecents = {'seabed', 'plains', 'archipelago', 'sky'}
	biome.stats.temperature = biomeRandom(-20, 30)
	biome.stats.nutrients = biomeRandom(20000, 150000)
	biome.stats.biomass = biomeRandom(10000, 90000)
	biome.stats.biomassDecay = biomeRandom(0.05, 0.08)
	biome.stats.biomeLoss = biomeRandom(0.1, 0.3)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0.4, 1.2)
	biome.stats.mutability = biomeRandom(0, 30)
	biome.stats.mobilityRestriction = biomeRandom(0, 4)

	return biome
end

generateBiome.plains = function ()
	local biome = initBiome()

	biome.name = 'Plains'
	biome.type = 'plains'
	biome.background = 'background-tree'
	biome.environmentTypes = {'air', 'land'}
	biome.adjecents = {'seashore', 'river', 'sky'}
	biome.stats.temperature = biomeRandom(-40, 5)
	biome.stats.nutrients = biomeRandom(10000, 100000)
	biome.stats.biomass = biomeRandom(4000, 30000)
	biome.stats.biomassDecay = biomeRandom(0.05, 0.08)
	biome.stats.biomeLoss = biomeRandom(0, 0.1)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0.6, 0.7)
	biome.stats.mutability = biomeRandom(0, 20)
	biome.stats.mobilityRestriction = biomeRandom(0, 5)

	return biome
end

generateBiome.lake = function ()
	local biome = initBiome()

	biome.name = 'Lake'
	biome.type = 'lake'
	biome.background = 'background-fluid'
	biome.environmentTypes = {'water', 'air', 'land'}
	biome.adjecents = {'plains', 'sky', 'river', 'hills', 'mountains', 'sky'}
	biome.stats.temperature = biomeRandom(-20, 30)
	biome.stats.nutrients = biomeRandom(20000, 150000)
	biome.stats.biomass = biomeRandom(10000, 90000)
	biome.stats.biomassDecay = biomeRandom(0.05, 0.08)
	biome.stats.biomeLoss = biomeRandom(0.1, 0.3)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0.4, 1.2)
	biome.stats.mutability = biomeRandom(0, 30)
	biome.stats.mobilityRestriction = biomeRandom(0, 4)

	return biome
end

generateBiome.archipelago = function ()
	local biome = initBiome()

	biome.name = 'Archipelago'
	biome.type = 'archipelago'
	biome.background = 'background-tree'
	biome.environmentTypes = {'water', 'air', 'land'}
	biome.adjecents = {'seashore', 'seasurface', 'sky'}
	biome.stats.temperature = biomeRandom(5, 30)
	biome.stats.nutrients = biomeRandom(20000, 100000)
	biome.stats.biomass = biomeRandom(3000, 60000)
	biome.stats.biomassDecay = biomeRandom(0.02, 0.03)
	biome.stats.biomeLoss = biomeRandom(0.1, 0.3)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0.6, 1)
	biome.stats.mutability = biomeRandom(20, 80)
	biome.stats.mobilityRestriction = biomeRandom(2, 5)

	return biome
end

generateBiome.sky = function ()
	local biome = initBiome()

	biome.name = 'Sky'
	biome.type = 'sky'
	biome.background = 'background-air'
	biome.environmentTypes = {'air'}
	biome.adjecents = {'hills', 'seashore', 'plains', 'mountains', 'caves', 'seasurface', 'archipelago', 'lake'}
	biome.stats.temperature = biomeRandom(-10, 20)
	biome.stats.nutrients = 0
	biome.stats.biomass = biomeRandom(1000, 10000)
	biome.stats.biomassDecay = biomeRandom(0, 0.1)
	biome.stats.biomeLoss = biomeRandom(0.9, 1)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0.8, 1)
	biome.stats.mutability = biomeRandom(30, 50)
	biome.stats.mobilityRestriction = biomeRandom(0, 6)

	return biome
end

generateBiome.hills = function ()
	local biome = initBiome()

	biome.name = 'Hills'
	biome.type = 'hills'
	biome.environmentTypes = {'air', 'land'}
	biome.adjecents = {'seashore', 'mountains', 'plains', 'sky', 'lake'}
	biome.stats.temperature = biomeRandom(-20, 30)
	biome.stats.nutrients = biomeRandom(20000, 100000)
	biome.stats.biomass = biomeRandom(8000, 60000)
	biome.stats.biomassDecay = biomeRandom(0.02, 0.05)
	biome.stats.biomeLoss = biomeRandom(0, 0.2)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0.4, 0.7)
	biome.stats.mutability = biomeRandom(0, 30)
	biome.stats.mobilityRestriction = biomeRandom(3, 8)

	return biome
end

generateBiome.caves = function ()
	local biome = initBiome()

	biome.name = 'Caves'
	biome.type = 'caves'
	biome.environmentTypes = {'air', 'water', 'land'}
	biome.adjecents = {'seashore', 'mountains', 'hills', 'sky'}
	biome.stats.temperature = biomeRandom(-10, 20)
	biome.stats.nutrients = biomeRandom(10000, 80000)
	biome.stats.biomass = biomeRandom(7000, 40000)
	biome.stats.biomassDecay = biomeRandom(0.02, 0.06)
	biome.stats.biomeLoss = biomeRandom(0, 0.1)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0, 0.1)
	biome.stats.mutability = biomeRandom(10, 20)
	biome.stats.mobilityRestriction = biomeRandom(4, 15)

	return biome
end


generateBiome.mountains = function ()
	local biome = initBiome()

	biome.name = 'Mountains'
	biome.type = 'mountains'
	biome.background = 'background-air'
	biome.environmentTypes = {'air', 'land'}
	biome.adjecents = {'hills', 'river', 'sky', 'lake'}
	biome.stats.temperature = biomeRandom(-30, 20)
	biome.stats.nutrients = biomeRandom(10000, 40000)
	biome.stats.biomass = biomeRandom(8000, 30000)
	biome.stats.biomassDecay = biomeRandom(0.02, 0.04)
	biome.stats.biomeLoss = biomeRandom(0.2, 0.4)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0.5, 0.7)
	biome.stats.mutability = biomeRandom(0, 30)
	biome.stats.mobilityRestriction = biomeRandom(4, 12)

	return biome
end

generateBiome.river = function ()
	local biome = initBiome()

	biome.name = 'River'
	biome.type = 'river'
	biome.background = 'background-fluid'
	biome.environmentTypes = {'water', 'air', 'land'}
	biome.adjecents = {'seashore', 'hills', 'mountains', 'sky', 'lake'}
	biome.stats.temperature = biomeRandom(-30, 40)
	biome.stats.nutrients = biomeRandom(30000, 120000)
	biome.stats.biomass = biomeRandom(10000, 80000)
	biome.stats.biomassDecay = biomeRandom(0.02, 0.03)
	biome.stats.biomeLoss = biomeRandom(0.1, 0.3)
	biome.stats.detriusmass = biome.stats.biomass * biome.stats.biomassDecay * biomeRandom(0.5, 1.5)
	biome.stats.luminosity = biomeRandom(0.3, 0.7)
	biome.stats.mutability = biomeRandom(0, 20)
	biome.stats.mobilityRestriction = biomeRandom(2, 8)

	return biome
end
