function initSpecies (name, speciesLayer)
	local species = {}

	species.prop = MOAIProp2D.new ()

	species.name = name

	speciesNameBox:setString ( '<c:000>' .. species.name .. '<c>' )

	species.dnaStats = {}
	species.dnaStats.airMobility = 0
	species.dnaStats.landMobility = 0
	species.dnaStats.waterMobility = 0
	species.dnaStats.reproduction = 0
	species.dnaStats.awareness = 0
	species.dnaStats.predation = 0
	species.dnaStats.antiPredation = 0
	species.dnaStats.mutability = 0
	species.dnaStats.photosynthesis = 0
	species.dnaStats.detrivoricity = 0
	species.dnaStats.digestion = 0
	species.dnaStats.consumption = 0

	species.dnaStats.temperatureMod = 0
	species.dnaStats.nutrientAbsorbtion = 0
	species.dnaStats.aquaOxygenation = 0
	species.dnaStats.aeroOxygenation = 0

	species.dnaStats.size = 10
	species.population = 1000
	species.mortality = 0
	species.biomass = 100

	species.dnaStats.consumption = 5
	species.intake = 0

	species.mutations = 0

	species.generation = 0

	species.dna = {}

	species.start = function (this)
		local body = initDna('Body')
		body.size = 5
		body.consumption = 1
		body.reproduction = 10
		body.detrivoricity = 5
		this:addDna(body)
	end

	species.delete = function (this)
		speciesLayer:removeProp(this.prop)

		for k, dna in pairs(this.dna) do
			speciesLayer:removeProp(dna.prop)
			speciesLayer:removeProp(dna.nameBox)
			if (dna.line) then
				speciesLayer:removeProp(dna.line)
			end
		end
		if (extinctProp) then
			extinctProp:setVisible(false)
		end
	end

	species.update = function (this)

		for k, stat in pairs(this.dnaStats) do
			this.dnaStats[k] = 0
		end

		for k, dna in pairs(this.dna) do
			dna:update()
			for k, value in pairs(this.dnaStats) do
				if (dna[k]) then
					species.dnaStats[k] = species.dnaStats[k] + dna[k]
				end
			end
		end

		species.population = round(species.population)
		species.biomass = round(species.population * species.dnaStats.size)

		topBarTxt:setString ( '<c:000>Generation <c:980e0e>'.. species.generation ..'<c>   <c:000>Population <c:115ea5>'.. species.population ..'<c>   <c:000>Biomass <c:1b6a12>'.. species.biomass ..'<c>' )

		statBox.mutations:setString ( '<c:67e42f>'.. math.floor(species.mutations/2) ..'%' )

		if (sim.selectedDna == this) then
			this:select()
		end
	end

	species.select = function (this)
		if (sim.selectedDna and sim.selectedDna.image) then
			sim.selectedDna.prop:setDeck ( sim.selectedDna.image )
		end

		sim.selectedDna = this

		for k, value in pairs(this.dnaStats) do
			if (statBox[k]) then
				statBox[k]:setString('<c:74a7fb>' .. species.dnaStats[k])
			end
		end
		statBox['name']:setString('<c:fff>' .. species.name)
	end

	speciesLayer:insertProp( species.prop )

	species.addDna = function (this, dna)
		this.dna[#this.dna + 1] = dna
		dna.prop:setParent(this.prop)
		if (#this.dna > 1) then
			dna.parent = this.dna[1]
			dna.prop:setLoc(math.random(-200,200),math.random(-200,200))
		else
			dna.prop:setLoc(0, 0)
		end

		MOAIInputMgr.dnaHovers[#MOAIInputMgr.dnaHovers + 1] = dna.prop
		MOAIInputMgr.hovers[dna.hoverNr] = nil

		speciesLayer:insertProp(dna.prop)
		speciesLayer:insertProp(dna.nameBox)
		dna.species = this
		species:update()
	end

	-- Apply all generation effects that happen regardless
	species.nextGen = function (this)
		if (species.population > 0) then
			species.generation = species.generation + 1
			species.startPopulation = species.population
			species.energy = 0
			species.mortality = 0
			species.mutations = species.mutations + species.dnaStats.mutability

			if (species.mutations > 200) then
				species.mutations = species.mutations - 200
				sim.genepool:addDna(generateRandomDna())
			end
		end
	end

	-- Apply all generation effects that happen regardless
	species.applyBiome = function (this, biome)
		if (species.population <= 0) then
			return
		end


		--species.breedMass = species.population * (species.dnaStats.reproduction/100) * species.consumption

		species.mutations = species.mutations + biome.stats.mutability

		local maxNutrientIntake = species.dnaStats.photosynthesis/10 * biome.stats.luminosity * species.population
		local maxDetriusIntake = species.dnaStats.detrivoricity/10 * species.population

		local nutrientIntake = math.min(maxNutrientIntake, biome.stats.nutrients)
		local detriusIntake = math.min(maxDetriusIntake, biome.stats.detriusmass)

		species.energy = species.energy + nutrientIntake * 10
		species.energy = species.energy + detriusIntake * 10

		-- Eat nutrients, detrius
		biome.stats.nutrients = biome.stats.nutrients - nutrientIntake
		biome.stats.detriusmass = biome.stats.detriusmass - detriusIntake

		-- Consume for survival, poop to detrius/nutrients
		local survivalEnergy = species.dnaStats.consumption * species.population
		local consumedEnergy = math.min(species.energy, survivalEnergy)
		local consumedEnergyPerPop = consumedEnergy / species.population
		if (detriusIntake+detriusIntake > 0) then
			biome.stats.detriusmass = biome.stats.detriusmass + consumedEnergy/10 * nutrientIntake/(detriusIntake+detriusIntake)
			biome.stats.nutrients = biome.stats.nutrients + consumedEnergy/10 * detriusIntake/(detriusIntake+detriusIntake)
		end
		species.energy = species.energy - survivalEnergy

		local energyPerPop = species.dnaStats.size * 10
		if (species.energy >= 0) then
			-- Got leftover energy! breed
			local maxBreed = species.population * species.dnaStats.reproduction/100

			local breed = math.min(math.floor(maxBreed), math.floor(species.energy / energyPerPop))

			-- Energy to own biomass
			species.population = species.population + breed
			species.energy = species.energy - breed*energyPerPop

			-- Rest energy to detrius/nutrients
			if (detriusIntake+detriusIntake > 0) then
				biome.stats.detriusmass = biome.stats.detriusmass + species.energy/10 * nutrientIntake/(detriusIntake+detriusIntake)
				biome.stats.nutrients = biome.stats.nutrients + species.energy/10 * detriusIntake/(detriusIntake+detriusIntake)
			end
		else
			-- Time to start dying
			local survival = (species.energy+survivalEnergy) / survivalEnergy
			local starvation = species.population * (1 - survival)
			species.population = species.population - math.ceil(starvation)

			-- Dead pop becomes detrius
			biome.stats.detriusmass = biome.stats.detriusmass + starvation * species.dnaStats.size
		end

		if (species.population == 0) then
			if (not extinctProp) then
				extinctProp = MOAIProp2D.new ()
				extinctProp.img = MOAIGfxQuad2D.new ()
				extinctProp.img:setTexture ( "img/extinct.png" )
				extinctProp.img:setRect ( 0, 0, 256, 256 )
				extinctProp:setDeck ( extinctProp.img )
				extinctProp:setLoc(-365, -45)
				extinctProp:setPriority(10)
				layer:insertProp(extinctProp)
			end
			extinctProp:setVisible(true)
		end

		-- Scorekeepin
		if (not scoreBox.topBiomass.score or scoreBox.topBiomass.score < species.biomass) then
			scoreBox.topBiomass:setString(''.. math.floor(species.biomass))
			scoreBox.topBiomass.score = species.biomass
		end
		if (not scoreBox.topPopulation.score or scoreBox.topPopulation.score < species.population) then
			scoreBox.topPopulation:setString(''.. math.floor(species.population))
			scoreBox.topPopulation.score = species.population
		end
		if (not scoreBox.topEnergy.score or scoreBox.topEnergy.score < consumedEnergyPerPop) then
			scoreBox.topEnergy:setString(''.. math.floor(consumedEnergyPerPop))
			scoreBox.topEnergy.score = consumedEnergyPerPop
		end
		local totalScore = consumedEnergyPerPop * species.population * species.dnaStats.size / 100
		if (not scoreBox.total.score or scoreBox.total.score < totalScore) then
			scoreBox.total:setString(''.. math.floor(totalScore))
			scoreBox.total.score = totalScore
		end
	end

	species.cleanUp = function (this, biome)
	end

	species.doPredation = function (this, prey)
		local huntSuccess = 1
		-- Can they outrun you?
		huntSuccess = huntSuccess * math.min(1, prey.dnaStats.mobility / species.dnaStats.mobility)

		-- Can they outfight you?
		huntSuccess = huntSuccess * math.min(1, species.dnaStats.predation / prey.dnaStats.antiPredation)

		-- Hunt mass
		local totalPotentialHuntMass = (huntSuccess * prey.dnaStats.biomass) / species.population

		-- How hungry are we?
		local requiredEnergy = species.dnaStats.consumption - species.energy

		-- How much can we get out of the corpse?
		local totalPotentialHuntEnergy = math.min(1, totalPotentialHuntMass / (species.dnsStats.digestion / 100))

		-- How much do we need to hunt?
		local huntRequired = math.min(1, requiredEnergy / totalPotentialHuntEnergy)

		-- Will prey kill you?
		local huntDeath = huntSuccess * huntRequired * (prey.dnaStats.antiPredation / (species.dnaStats.predation + species.dnaStats.antiPredation))

		-- Effect changes
		species.population = species.population * huntDeath
		species.energy = species.energy + totalPotentialHuntEnergy * huntRequired
		prey.population = prey.population - (huntSuccess * huntRequired)
	end

	return species
end