dnaData = false
function initDna (name, species)
	local dna = {}

	if (name == nil) then
		name = 'DNA #'.. math.random(1,1000)
	end

	dna.name = name

	dna.size = 0
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = 0
	dna.reproduction = 0
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0

	dna.temperatureMod = 0

	dna.mutability = 0

	dna.photosynthesis = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0
	dna.detrivoricity = 0

	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	dna.consumption = 0

	dna.vestigial = false

	if (dnaData) then
		return dna
	end

	dna.parent = nil

	dna.image = MOAIGfxQuad2D.new ()
	dna.image:setTexture ( "img/dnadot.png" )
	dna.image:setRect ( -16, -16, 16, 16 )
	dna.selectImage = MOAIGfxQuad2D.new ()
	dna.selectImage:setTexture ( "img/dnadot-red.png" )
	dna.selectImage:setRect ( -16, -16, 16, 16 )

	dna.prop = MOAIProp2D.new ()
	dna.prop:setDeck ( dna.image )


	if (species) then
		dna.species = species
		dna.prop:setParent(species.prop)
		dna.prop:setLoc(math.random(-200,200),math.random(-200,200))
	end

	local textbox = MOAITextBox.new ()
	textbox:setString ( dna.name )
	textbox:setFont ( font )
	textbox:setTextSize ( 20 )
	textbox:setRect ( -75, 0, 75, 30 )
	textbox:setAlignment ( MOAITextBox.CENTER_JUSTIFY )
	textbox:setYFlip ( true )
	textbox:setParent (dna.prop)
	textbox:setLoc ( 0, 20 )
	textbox:setPriority ( 1000 )
	dna.nameBox = textbox

	dna.update = function (this)
		this.prop:setScl(math.max(1, this.size/5))
		this.nameBox:setString(this.name)

		if (dna.parent) then
			if (not dna.line) then
				dna.lineImg = MOAIGfxQuad2D.new ()
				dna.lineImg:setTexture ( "img/line.png" )

				dna.line = MOAIProp2D.new ()
				dna.line:setDeck( dna.lineImg )
				dna.prop:forceUpdate()
				dna.parent.prop:forceUpdate()
				dnaViewLayer:insertProp( dna.line )
			end

			dna.line:setLoc(dna.prop:getWorldLoc() )
			dna.line:setPriority(-1)
			local fromX, fromY = dna.prop:getWorldLoc()
			local toX, toY = dna.parent.prop:getWorldLoc()
			local distance = math.sqrt(math.pow(toX-fromX,2) + math.pow(toY-fromY,2))
			local rot = math.deg(math.atan2((toY-fromY), (toX-fromX)))
			dna.lineImg:setRect( 0, 2, distance, -2 )
			dna.line:setRot(rot)
		end
	end

	dna.select = function (this)
		if (sim.selectedDna and sim.selectedDna.image) then
			sim.selectedDna.prop:setDeck( sim.selectedDna.image )
		end

		if (sim.selectedDna == this) then
			sim.selectedDna = nil
			sim.species:select()
			return nil
		end

		dna.prop:setDeck ( dna.selectImage )

		sim.selectedDna = dna

		for k, value in pairs(this) do
			if (statBox[k]) then
				if (k == 'name') then
					statBox[k]:setString('<c:fff>DNA: ' .. value)
				else
					statBox[k]:setString('<c:74a7fb>' .. value)
				end
			end
		end
	end

	dna.setVisible = function(this, visible)
		local parent = this.prop:getAttrLink(MOAITransform2D.INHERIT_TRANSFORM)
		if (not visible) then
			this.prop:setParent(nil)
			this.prop:setAttrLink(MOAIProp.INHERIT_TRANSFORM, parent, MOAIProp.TRANSFORM_TRAIT)
			this.prop:setVisible(false)
		else
			this.prop:setAttrLink(MOAIProp.INHERIT_TRANSFORM, parent, MOAIProp.TRANSFORM_TRAIT)
			this.prop:setVisible(true)
		end
	end

	if (species) then
		dnaViewLayer:insertProp( dna.prop )
		dnaViewLayer:insertProp ( dna.nameBox )
		MOAIInputMgr.dnaHovers[#MOAIInputMgr.dnaHovers + 1] = dna.prop
	else
		layer:insertProp( dna.prop )
		layer:insertProp ( dna.nameBox )
		dna.hoverNr = #MOAIInputMgr.hovers + 1
		MOAIInputMgr.hovers[#MOAIInputMgr.hovers + 1] = dna.prop
	end

	dna.prop.hoverIn = function(self)
		dna:select()
	end
	dna.prop.hoverOut = function(self)
		sim.species:select()
	end


	return dna
end

generateRandomDna = function ()
	local randomGenList = {}
	for k, type in pairs(generateDna) do
		randomGenList[#randomGenList + 1] = k
	end

	return generateDna[randomGenList[math.random(1, #randomGenList)]]()
end

generateTypeDna = function (type)
	local randomGenList = {}
	dnaData = true
	for k, generator in pairs(generateDna) do
		local dna = generator()
		if (dna.type == type) then
			randomGenList[#randomGenList + 1] = k
		end
	end
	dnaData = false

	return generateDna[randomGenList[math.random(1, #randomGenList)]]()
end


generateDna = {}
generateDna.basicbody = function ()
	local dna = initDna()

	dna.name = 'Body'
	dna.type = 'body'
	dna.size = math.random(3, 8)
	dna.mobility = 0
	dna.storage = dna.size * biomeRandom(0.8, 1.3)
	dna.reproduction = math.random(1, 10)
	dna.consumption = math.ceil((dna.size * 0.5 + dna.reproduction * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end

generateDna.head = function ()
	local dna = initDna()

	dna.name = 'Head'
	dna.type = 'limb'
	dna.size = math.random(1, 4)
	dna.storage = dna.size * 0.5 * biomeRandom(0.8, 1.3)
	dna.mobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5 + dna.reproduction * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = math.random(1, 3)
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end

generateDna.gills = function ()
	local dna = initDna()

	dna.name = 'Gills'
	dna.type = 'intestine'
	dna.size = math.random(1, 2)
	dna.mobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5 + dna.reproduction * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 1
	dna.aeroOxygenation = 0

	return dna
end

generateDna.gills = function ()
	local dna = initDna()

	dna.name = 'Lungs'
	dna.type = 'intestine'
	dna.size = math.random(1, 2)
	dna.mobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5 + dna.reproduction * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 1

	return dna
end

generateDna.stomach = function ()
	local dna = initDna()

	dna.name = 'Stomach'
	dna.type = 'intestine'
	dna.size = math.random(1, 2)
	dna.mobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.7) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = math.random(1, 4)
	dna.digestion = math.random(7, 20)
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 1

	return dna
end


generateDna.legs = function ()
	local dna = initDna()

	dna.name = 'Set o Legs'
	dna.type = 'limbs'
	dna.size = math.random(3, 4)
	dna.landMobility = math.random(3, 9)
	dna.waterMobility = math.random(1, 3)
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = math.random(0, 3)
	dna.antiPredation = math.random(0, 2)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end


generateDna.featherwings = function ()
	local dna = initDna()

	dna.name = 'Feathered wings'
	dna.type = 'limbs'
	dna.size = math.random(3, 4)
	dna.landMobility = math.random(1, 4)
	dna.waterMobility = 0
	dna.airMobility = math.random(5, 10)
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = math.random(0, 1)
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end


generateDna.arms = function ()
	local dna = initDna()

	dna.name = 'Arms'
	dna.type = 'limbs'
	dna.size = math.random(2, 5)
	dna.landMobility = math.random(1, 2)
	dna.waterMobility = math.random(3, 6)
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = math.random(1, 4)
	dna.antiPredation = math.random(0, 2)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end

generateDna.fins = function ()
	local dna = initDna()

	dna.name = 'Fins'
	dna.type = 'limbs'
	dna.size = math.random(1, 3)
	dna.landMobility = math.random(0, 2)
	dna.waterMobility = math.random(3, 8)
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end

generateDna.fangs = function ()
	local dna = initDna()

	dna.name = 'Fangs'
	dna.type = 'close-weaponry'
	dna.size = math.random(1, 2)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.reproduction = math.floor(math.random(1, 6)/6)
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = math.random(3, 4)
	dna.antiPredation = math.random(1, 2)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = math.random(1,2)
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end

generateDna.claws = function ()
	local dna = initDna()

	dna.name = 'Claws'
	dna.type = 'limb-weaponry'
	dna.size = math.random(1, 2)
	dna.landMobility = math.random(1,3)
	dna.waterMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = math.random(3, 5)
	dna.antiPredation = math.random(1, 3)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end

generateDna.poisoned = function ()
	local dna = initDna()

	dna.name = 'Poisoned tissue'
	dna.type = 'modifier'
	dna.size = math.random(1, 2)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = math.random(5, 10)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end

generateDna.poison = function ()
	local dna = initDna()

	dna.name = 'Poison'
	dna.type = 'modifier'
	dna.size = math.random(1, 3)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = math.random(3, 10)
	dna.antiPredation = math.random(2 ,4)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = 0
	dna.aeroOxygenation = 0

	return dna
end

generateDna.leafs = function ()
	local dna = initDna()

	dna.name = 'Leafs'
	dna.type = 'limbs'
	dna.size = math.random(3, 5)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = math.random(7, 10)
	dna.mutability = 0
	dna.detrivoricity = 0
	dna.digestion = 0
	dna.aquaOxygenation = true
	dna.aeroOxygenation = true

	return dna
end


generateDna.roots = function ()
	local dna = initDna()

	dna.name = 'Roots'
	dna.type = 'limbs'
	dna.size = math.random(3, 5)
	dna.storage = dna.size * biomeRandom(0.8, 1.3)
	dna.landMobility = -10
	dna.waterMobility = -10
	dna.airMobility = -10
	dna.reproduction = math.random(1,4)
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = math.random(2, 3)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = math.random(3,20)
	dna.digestion = 0

	return dna
end


generateDna.eyes = function ()
	local dna = initDna()

	dna.name = 'Pair o eyes'
	dna.type = 'sensor'
	dna.size = math.random(1, 2)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.9) * biomeRandom(0.8, 1,1))
	dna.awareness = math.random(1, 5)
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.ears = function ()
	local dna = initDna()

	dna.name = 'Pair o ears'
	dna.type = 'sensor'
	dna.size = math.random(1, 2)
	dna.landMobility = 0
	dna.temperatureMod = math.random(0, 5)
	dna.waterMobility = 0
	dna.airMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.9) * biomeRandom(0.8, 1,1))
	dna.awareness = math.random(2, 7)
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.nose = function ()
	local dna = initDna()

	dna.name = 'Nose'
	dna.type = 'sensor'
	dna.size = math.random(1, 2)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.9) * biomeRandom(0.8, 1,1))
	dna.awareness = math.random(2, 7)
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.anteannea = function ()
	local dna = initDna()

	dna.name = 'Antennae'
	dna.type = 'sensor'
	dna.size = math.random(1, 2)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.2) * biomeRandom(0.8, 1,1))
	dna.awareness = math.random(1, 4)
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.camouflage = function ()
	local dna = initDna()

	dna.name = 'Camouflage'
	dna.type = 'modifier'
	dna.size = math.random(1, 2)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.8) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = math.random(1,3)
	dna.antiPredation = math.random(4,8)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.fur = function ()
	local dna = initDna()

	dna.name = 'Fur'
	dna.type = 'modifier'
	dna.size = math.random(1, 2)
	dna.temperatureMod = math.random(5, 15)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.8) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = math.random(2, 4)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.scales = function ()
	local dna = initDna()

	dna.name = 'Scales'
	dna.type = 'modifier'
	dna.size = math.random(1, 2)
	dna.temperatureMod = math.random(4, 10)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.8) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = math.random(2, 8)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.beak = function ()
	local dna = initDna()

	dna.name = 'Beak'
	dna.type = 'limb'
	dna.size = math.random(1, 3)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.3) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = math.random(1, 6)
	dna.antiPredation = math.random(0, 2)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = math.random(1,3)

	return dna
end

generateDna.trunk = function ()
	local dna = initDna()

	dna.name = 'Trunk'
	dna.type = 'limb'
	dna.size = math.random(2, 5)
	dna.landMobility = 0
	dna.waterMobility = math.random(0, 3)
	dna.airMobility = 0
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.3) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = math.random(1, 6)
	dna.antiPredation = math.random(0, 2)
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.skinwings = function ()
	local dna = initDna()

	dna.name = 'Skin wings'
	dna.type = 'limb'
	dna.size = math.random(1, 5)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = math.random(3, 8)
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.membranewings = function ()
	local dna = initDna()

	dna.name = 'Membrane wings'
	dna.type = 'limb'
	dna.size = math.random(1, 2)
	dna.landMobility = 0
	dna.waterMobility = 0
	dna.airMobility = math.random(2, 5)
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end

generateDna.feathers = function ()
	local dna = initDna()

	dna.name = 'Feathers'
	dna.type = 'modifier'
	dna.size = math.random(1, 2)
	dna.temperatureMod = math.random(10, 20)
	dna.landMobility = 0
	dna.waterMobility = math.random(1, 2)
	dna.airMobility = math.random(1, 2)
	dna.reproduction = 0
	dna.consumption = math.ceil((dna.size * 0.5) * biomeRandom(0.8, 1,1))
	dna.awareness = 0
	dna.predation = 0
	dna.antiPredation = 0
	dna.photosynthesis = 0
	dna.mutability = 0
	dna.nutrientAbsorbtion = 0
	dna.digestion = 0

	return dna
end